{% extends "base.html" %}

{% block title %}Input form{% endblock %}

{% block content %}
<div class="container">
  <h1>{{ title }}</h1>
  {% block preform %}
  {% endblock preform %}
  {% import "uxfab/wtf.html" as wtf %}
  {{ wtf.quick_form(form) }}
  {% block postpreform %}
  {% endblock postpreform %}
<hr>
... or <a href="{{ url_for('iam_blueprint.register') }}">register</a>
</div>
{% endblock content %}
